import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSAttributedString(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSFontAttributeName, str)
        self.assertIsInstance(AppKit.NSParagraphStyleAttributeName, str)
        self.assertIsInstance(AppKit.NSForegroundColorAttributeName, str)
        self.assertIsInstance(AppKit.NSSuperscriptAttributeName, str)
        self.assertIsInstance(AppKit.NSBackgroundColorAttributeName, str)
        self.assertIsInstance(AppKit.NSLigatureAttributeName, str)
        self.assertIsInstance(AppKit.NSBaselineOffsetAttributeName, str)
        self.assertIsInstance(AppKit.NSKernAttributeName, str)
        self.assertIsInstance(AppKit.NSLinkAttributeName, str)

        self.assertIsInstance(AppKit.NSStrokeWidthAttributeName, str)
        self.assertIsInstance(AppKit.NSStrokeColorAttributeName, str)
        self.assertIsInstance(AppKit.NSUnderlineColorAttributeName, str)
        self.assertIsInstance(AppKit.NSStrikethroughStyleAttributeName, str)
        self.assertIsInstance(AppKit.NSUnderlineStyleAttributeName, str)
        self.assertIsInstance(AppKit.NSObliquenessAttributeName, str)
        self.assertIsInstance(AppKit.NSStrikethroughColorAttributeName, str)
        self.assertIsInstance(AppKit.NSShadowAttributeName, str)
        self.assertIsInstance(AppKit.NSExpansionAttributeName, str)
        self.assertIsInstance(AppKit.NSToolTipAttributeName, str)

        self.assertIsInstance(AppKit.NSCharacterShapeAttributeName, str)
        self.assertIsInstance(AppKit.NSGlyphInfoAttributeName, str)
        self.assertIsInstance(AppKit.NSMarkedClauseSegmentAttributeName, str)

        self.assertEqual(AppKit.NSUnderlineStyleNone, 0x00)
        self.assertEqual(AppKit.NSUnderlineStyleSingle, 0x01)
        self.assertEqual(AppKit.NSUnderlineStyleThick, 0x02)
        self.assertEqual(AppKit.NSUnderlineStyleDouble, 0x09)

        self.assertEqual(AppKit.NSUnderlinePatternSolid, 0x0000)
        self.assertEqual(AppKit.NSUnderlinePatternDot, 0x0100)
        self.assertEqual(AppKit.NSUnderlinePatternDash, 0x0200)
        self.assertEqual(AppKit.NSUnderlinePatternDashDot, 0x0300)
        self.assertEqual(AppKit.NSUnderlinePatternDashDotDot, 0x0400)

        self.assertEqual(AppKit.NSUnderlineByWordMask, 0x8000)
        self.assertEqual(AppKit.NSWritingDirectionEmbedding, 0 << 1)
        self.assertEqual(AppKit.NSWritingDirectionOverride, 1 << 1)

        self.assertIsInstance(AppKit.NSSpellingStateAttributeName, str)

        self.assertEqual(AppKit.NSSpellingStateSpellingFlag, (1 << 0))
        self.assertEqual(AppKit.NSSpellingStateGrammarFlag, (1 << 1))

        self.assertIsInstance(AppKit.NSPlainTextDocumentType, str)
        self.assertIsInstance(AppKit.NSRTFTextDocumentType, str)
        self.assertIsInstance(AppKit.NSRTFDTextDocumentType, str)
        self.assertIsInstance(AppKit.NSMacSimpleTextDocumentType, str)
        self.assertIsInstance(AppKit.NSHTMLTextDocumentType, str)
        self.assertIsInstance(AppKit.NSDocFormatTextDocumentType, str)
        self.assertIsInstance(AppKit.NSWordMLTextDocumentType, str)
        self.assertIsInstance(AppKit.NSWebArchiveTextDocumentType, str)
        self.assertIsInstance(AppKit.NSOfficeOpenXMLTextDocumentType, str)
        self.assertIsInstance(AppKit.NSOpenDocumentTextDocumentType, str)

        self.assertIsInstance(AppKit.NSPaperSizeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSLeftMarginDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSRightMarginDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSTopMarginDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSBottomMarginDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSViewSizeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSViewZoomDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSViewModeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSDocumentTypeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSReadOnlyDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSConvertedDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCocoaVersionDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSBackgroundColorDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSHyphenationFactorDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSDefaultTabIntervalDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCharacterEncodingDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSTitleDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCompanyDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCopyrightDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSSubjectDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSAuthorDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSKeywordsDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCommentDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSEditorDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCreationTimeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSModificationTimeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSExcludedElementsDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSTextEncodingNameDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSPrefixSpacesDocumentAttribute, str)

        self.assertIsInstance(AppKit.NSDocumentTypeDocumentOption, str)
        self.assertIsInstance(AppKit.NSDefaultAttributesDocumentOption, str)
        self.assertIsInstance(AppKit.NSCharacterEncodingDocumentOption, str)
        self.assertIsInstance(AppKit.NSTextEncodingNameDocumentOption, str)
        self.assertIsInstance(AppKit.NSBaseURLDocumentOption, str)
        self.assertIsInstance(AppKit.NSTimeoutDocumentOption, str)
        self.assertIsInstance(AppKit.NSWebPreferencesDocumentOption, str)
        self.assertIsInstance(AppKit.NSWebResourceLoadDelegateDocumentOption, str)
        self.assertIsInstance(AppKit.NSTextSizeMultiplierDocumentOption, str)

        self.assertEqual(AppKit.NSNoUnderlineStyle, 0)
        self.assertEqual(AppKit.NSSingleUnderlineStyle, 1)

        self.assertIsInstance(AppKit.NSUnderlineStrikethroughMask, int)

        self.assertIsInstance(AppKit.NSAttachmentAttributeName, str)
        self.assertIsInstance(AppKit.NSCursorAttributeName, str)

        self.assertEqual(AppKit.NSUnderlineStylePatternSolid, 0x0000)
        self.assertEqual(AppKit.NSUnderlineStylePatternDot, 0x0100)
        self.assertEqual(AppKit.NSUnderlineStylePatternDash, 0x0200)
        self.assertEqual(AppKit.NSUnderlineStylePatternDashDot, 0x0300)
        self.assertEqual(AppKit.NSUnderlineStylePatternDashDotDot, 0x0400)

        self.assertEqual(AppKit.NSUnderlineStyleByWord, 0x8000)

        self.assertEqual(AppKit.NSTextScalingStandard, 0)
        self.assertEqual(AppKit.NSTextScalingiOS, 1)

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(AppKit.NSAppearanceDocumentAttribute, str)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(AppKit.NSTextScalingDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSSourceTextScalingDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSTargetTextScalingDocumentOption, str)
        self.assertIsInstance(AppKit.NSSourceTextScalingDocumentOption, str)

    def testMethods(self):
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithURL_options_documentAttributes_error_, 2
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithURL_options_documentAttributes_error_, 3
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithData_options_documentAttributes_error_, 2
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithData_options_documentAttributes_error_, 3
        )

        self.assertArgIsOut(
            AppKit.NSAttributedString.dataFromRange_documentAttributes_error_, 2
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.fileWrapperFromRange_documentAttributes_error_, 2
        )

        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithPath_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithURL_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithRTF_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithRTFD_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithHTML_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithHTML_baseURL_documentAttributes_, 2
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithDocFormat_documentAttributes_, 1
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithHTML_options_documentAttributes_, 2
        )
        self.assertArgIsOut(
            AppKit.NSAttributedString.initWithRTFDFileWrapper_documentAttributes_, 1
        )

        self.assertResultIsBOOL(
            AppKit.NSMutableAttributedString.readFromURL_options_documentAttributes_error_
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromURL_options_documentAttributes_error_,
            2,
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromURL_options_documentAttributes_error_,
            3,
        )

        self.assertResultIsBOOL(
            AppKit.NSMutableAttributedString.readFromData_options_documentAttributes_error_
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromData_options_documentAttributes_error_,
            2,
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromData_options_documentAttributes_error_,
            3,
        )

        self.assertResultIsBOOL(
            AppKit.NSMutableAttributedString.readFromURL_options_documentAttributes_
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromURL_options_documentAttributes_, 2
        )
        self.assertResultIsBOOL(
            AppKit.NSMutableAttributedString.readFromData_options_documentAttributes_
        )
        self.assertArgIsOut(
            AppKit.NSMutableAttributedString.readFromData_options_documentAttributes_, 2
        )

        self.assertArgHasType(
            AppKit.NSAttributedString.URLAtIndex_effectiveRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )

        self.assertArgIsBOOL(
            AppKit.NSMutableAttributedString.nextWordFromIndex_forward_, 1
        )
        self.assertResultIsBOOL(AppKit.NSAttributedString.containsAttachments)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(
            AppKit.NSMutableAttributedString.containsAttachmentsInRange_
        )

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSManagerDocumentAttribute, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSWritingDirectionAttributeName, str)
        self.assertIsInstance(AppKit.NSFileTypeDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSCategoryDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSFileTypeDocumentOption, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSVerticalGlyphFormAttributeName, str)
        self.assertIsInstance(AppKit.NSTextLayoutSectionOrientation, str)
        self.assertIsInstance(AppKit.NSTextLayoutSectionRange, str)
        self.assertIsInstance(AppKit.NSTextLayoutSectionsAttribute, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSTextAlternativesAttributeName, str)
        self.assertIsInstance(AppKit.NSUsesScreenFontsDocumentAttribute, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(AppKit.NSTextEffectAttributeName, str)
        self.assertIsInstance(AppKit.NSTextEffectLetterpressStyle, str)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSDefaultAttributesDocumentAttribute, str)

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(AppKit.NSTrackingAttributeName, str)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSHapticFeedbackHelper(AppKit.NSObject):
    def performFeedbackPattern_performanceTime_(self, p, t):
        pass


class TestNSHapticFeedback(TestCase):
    @min_os_level("10.11")
    def testConstants(self):
        self.assertEqual(AppKit.NSHapticFeedbackPatternGeneric, 0)
        self.assertEqual(AppKit.NSHapticFeedbackPatternAlignment, 1)
        self.assertEqual(AppKit.NSHapticFeedbackPatternLevelChange, 2)

        self.assertEqual(AppKit.NSHapticFeedbackPerformanceTimeDefault, 0)
        self.assertEqual(AppKit.NSHapticFeedbackPerformanceTimeNow, 1)
        self.assertEqual(AppKit.NSHapticFeedbackPerformanceTimeDrawCompleted, 2)

    @min_os_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("NSHapticFeedbackPerformer")
        self.assertArgHasType(
            TestNSHapticFeedbackHelper.performFeedbackPattern_performanceTime_,
            1,
            objc._C_NSUInteger,
        )



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class XPCHelper(Foundation.NSObject):
    def remoteObjectProxyWithErrorHandler_(self, a):
        pass

    def listener_shouldAcceptNewConnection_(self, a, b):
        pass


class TestNSXPCConnection(TestCase):
    @min_os_level("10.8")
    def testProtocolObjects(self):
        objc.protocolNamed("NSXPCProxyCreating")
        objc.protocolNamed("NSXPCListenerDelegate")

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSXPCConnectionPrivileged, 1 << 12)

    @min_os_level("10.8")
    def testProtocol10_8(self):
        self.assertArgIsBlock(XPCHelper.remoteObjectProxyWithErrorHandler_, 0, b"v@")
        self.assertResultIsBOOL(XPCHelper.listener_shouldAcceptNewConnection_)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBlock(
            Foundation.NSXPCConnection.remoteObjectProxyWithErrorHandler_, 0, b"v@"
        )
        self.assertResultIsBlock(Foundation.NSXPCConnection.interruptionHandler, b"v")
        self.assertArgIsBlock(
            Foundation.NSXPCConnection.setInterruptionHandler_, 0, b"v"
        )
        self.assertResultIsBlock(Foundation.NSXPCConnection.invalidationHandler, b"v")
        self.assertArgIsBlock(
            Foundation.NSXPCConnection.setInvalidationHandler_, 0, b"v"
        )

        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.setClasses_forSelector_argumentIndex_ofReply_, 3
        )
        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.classesForSelector_argumentIndex_ofReply_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.setInterface_forSelector_argumentIndex_ofReply_, 3
        )
        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.interfaceForSelector_argumentIndex_ofReply_, 2
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBlock(
            Foundation.NSXPCConnection.synchronousRemoteObjectProxyWithErrorHandler_,
            0,
            b"v@",
        )

    @min_os_level("10.15")  # Not actually on 10.14...
    def testMethods10_14(self):
        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.setXPCType_forSelector_argumentIndex_ofReply_, 3
        )
        self.assertArgIsBOOL(
            Foundation.NSXPCInterface.XPCTypeForSelector_argumentIndex_ofReply_, 2
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSTextStorageHelper(AppKit.NSObject):
    def textStorage_willProcessEditing_range_changeInLength_(self, s, e, r, x):
        pass

    def textStorage_didProcessEditing_range_changeInLength_(self, s, e, r, x):
        pass

    def processEditingForTextStorage_edited_range_changeInLength_invalidateRange_(
        self, a, b, c, d, e
    ):
        pass

    def performEditingTransactionForTextStorage_usingBlock_(self, a, b):
        pass


class TestNSTextStorage(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTextStorageEditedAttributes, 1 << 0)
        self.assertEqual(AppKit.NSTextStorageEditedCharacters, 1 << 1)

        self.assertIsInstance(AppKit.NSTextStorageWillProcessEditingNotification, str)
        self.assertIsInstance(AppKit.NSTextStorageDidProcessEditingNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextStorage.fixesAttributesLazily)

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextStorageDelegate")

    @min_sdk_level("12.0")
    def testProtocolObjects12_0(self):
        objc.protocolNamed("NSTextStorageObserving")

    def testProtocols(self):
        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_willProcessEditing_range_changeInLength_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_willProcessEditing_range_changeInLength_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_willProcessEditing_range_changeInLength_,
            3,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_didProcessEditing_range_changeInLength_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_didProcessEditing_range_changeInLength_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.textStorage_didProcessEditing_range_changeInLength_,
            3,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSTextStorageHelper.processEditingForTextStorage_edited_range_changeInLength_invalidateRange_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.processEditingForTextStorage_edited_range_changeInLength_invalidateRange_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.processEditingForTextStorage_edited_range_changeInLength_invalidateRange_,
            3,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTextStorageHelper.processEditingForTextStorage_edited_range_changeInLength_invalidateRange_,
            4,
            AppKit.NSRange.__typestr__,
        )

        self.assertArgIsBlock(
            TestNSTextStorageHelper.performEditingTransactionForTextStorage_usingBlock_,
            1,
            b"v",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMenuToolbarItem(TestCase):
    @min_os_level("10.15")
    def test_nethods10_15(self):
        self.assertResultIsBOOL(AppKit.NSMenuToolbarItem.showsIndicator)
        self.assertArgIsBOOL(AppKit.NSMenuToolbarItem.setShowsIndicator_, 0)



import sys
import objc
import warnings

import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, cast_uint, onlyPython2


class TestNSString(TestCase):
    def testClassTree(self):
        self.assertTrue(issubclass(objc.pyobjc_unicode, str))

    def testCompare(self):
        self.assertTrue(
            Foundation.NSString.localizedCaseInsensitiveCompare_("foo", "bar") == 1,
            b"NSString doesn'.decode('ascii')t compare correctly",
        )
        self.assertTrue(
            Foundation.NSString.localizedCaseInsensitiveCompare_("foo", "Foo") == 0,
            b"NSString doesn'.decode('ascii')t compare correctly",
        )

    def testFormatting(self):
        # The test on instances is slightly more verbose to avoid warnings
        obj = Foundation.NSString.alloc().initWithFormat_("foo %d", 42)
        self.assertEqual(obj, "foo 42")

        obj = Foundation.NSString.alloc().initWithFormat_locale_("foo %d", {}, 42)
        self.assertEqual(obj, "foo 42")

    def testGetCString(self):
        # Custom wrappers
        v = Foundation.NSString.stringWithString_("hello world")

        self.assertEqual(v, "hello world")

        x = v.getCString_maxLength_(None, 16)
        self.assertEqual(x, b"hello world")

        self.assertRaises(objc.error, v.getCString_maxLength_, None, 4)

        x, loc = v.getCString_maxLength_range_remainingRange_(None, 4, (1, 4), None)
        self.assertEqual(x, b"ello")
        self.assertEqual(loc.location, 5)
        self.assertEqual(loc.length, 0)


class TestNSStringBridging(TestCase):
    def setUp(self):
        self.nsUniString = Foundation.NSString.stringWithString_("unifoo")
        self.pyUniString = "unifoo"

    def testBasicComparison(self):
        self.assertEqual("unifoo", Foundation.NSString.stringWithString_("unifoo"))

        u = "\xc3\xbc\xc3\xb1\xc3\xae\xc3\xa7\xc3\xb8d\xc3\xa8"
        self.assertEqual(u, Foundation.NSString.stringWithString_(u))

    def testTypesAndClasses(self):
        self.assertIsInstance(self.nsUniString, str)
        self.assertIsInstance(self.pyUniString, str)

    @onlyPython2
    def testStrConversion(self):
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore")

            curEnabledFlag = objc.getStrBridgeEnabled()
            objc.setStrBridgeEnabled(True)
            try:
                v = Foundation.NSString.stringWithString_("hello2")
                self.assertIsInstance(v, objc.pyobjc_unicode)
                self.assertEqual(v, "hello2")

                self.assertRaises(UnicodeError, str, "\xff")
                # XXX: string bridge now uses the default Foundation.NSString encoding
                # self.assertRaises(UnicodeError, Foundation.NSString.stringWithString_, '\xff')

                objc.setStrBridgeEnabled(False)

                warnings.filterwarnings("error", category=objc.PyObjCStrBridgeWarning)
                try:
                    # v = Foundation.NSString.stringWithString_("hello")

                    # we need to make sure that the str is unique
                    # because an already bridged one might have crossed
                    # and would be cached
                    newString = type("test_str", (str,), {})("hello2")
                    self.assertRaises(
                        objc.PyObjCStrBridgeWarning,
                        Foundation.NSString.stringWithString_,
                        newString,
                    )

                finally:
                    del warnings.filters[0]

            finally:
                objc.setStrBridgeEnabled(curEnabledFlag)

    def testNSStringMethodAccess(self):
        self.assertIsInstance(self.nsUniString, objc.pyobjc_unicode)
        v = self.nsUniString.stringByAppendingString_
        self.assertIsInstance(v, objc.selector)


class TestMutable(TestCase):
    def testSync(self):
        """
        Test that python and ObjC string representation are not
        automaticly synchronized.
        """
        pyStr = Foundation.NSMutableString.stringWithString_("hello")
        ocStr = pyStr.nsstring()
        self.assertEqual(pyStr, "hello")
        self.assertIsInstance(ocStr, Foundation.NSMutableString)
        ocStr.appendString_(" world")
        self.assertEqual(pyStr, "hello")


class TestPickle(TestCase):
    """
    Testcases for pickling of Objective-C strings. Those are pickled as
    str strings.
    """

    def setUp(self):
        self.strVal = Foundation.NSTaskDidTerminateNotification

    def testPickle(self):
        """
        Check that ObjC-strings pickle as str strings
        """
        import pickle

        s = pickle.dumps(self.strVal, 0)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

        s = pickle.dumps(self.strVal, 1)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

        s = pickle.dumps(self.strVal, 2)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

    @onlyPython2
    def testCPickle(self):
        """
        Check that ObjC-strings pickle as str strings
        """
        import cPickle as pickle

        s = pickle.dumps(self.strVal, 0)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

        s = pickle.dumps(self.strVal, 1)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

        s = pickle.dumps(self.strVal, 2)
        v = pickle.loads(s)
        self.assertEqual(type(v), str)

    def testFormat(self):
        v = self.strVal

        d = v.stringByAppendingFormat_("hello")
        self.assertEqual(d, v + "hello")

        d = v.stringByAppendingFormat_("hello %s %d", b"world", 101)
        self.assertEqual(d, v + "hello world 101")

        v = Foundation.NSString.alloc().initWithFormat_("%s %d %s", b"a", 44, b"cc")
        self.assertEqual(v, "a 44 cc")

        v = Foundation.NSString.alloc().initWithFormat_locale_(
            "%s %d %s", None, b"a", 44, b"cc"
        )
        self.assertEqual(v, "a 44 cc")

        v = Foundation.NSString.stringWithFormat_("aap %s mies", b"noot")
        self.assertEqual(v, "aap noot mies")

        v = Foundation.NSString.localizedStringWithFormat_("aap %s mies", b"noot")
        self.assertEqual(v, "aap noot mies")

        v = Foundation.NSMutableString.stringWithString_("hello")
        v.appendFormat_(" bar %s", b"baz")
        self.assertEqual(v.nsstring(), "hello bar baz")

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSRegularExpressionSearch, 1024)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(Foundation.NSMaximumStringLength, sys.maxsize)
        self.assertEqual(Foundation.NSDiacriticInsensitiveSearch, 128)
        self.assertEqual(Foundation.NSWidthInsensitiveSearch, 256)
        self.assertEqual(Foundation.NSForcedOrderingSearch, 512)

        self.assertEqual(Foundation.NSProprietaryStringEncoding, 65536)

    def testConstants(self):
        self.assertIsInstance(Foundation.NSParseErrorException, str)
        self.assertIsInstance(Foundation.NSCharacterConversionException, str)

        self.assertEqual(Foundation.NSCaseInsensitiveSearch, 1)
        self.assertEqual(Foundation.NSLiteralSearch, 2)
        self.assertEqual(Foundation.NSBackwardsSearch, 4)
        self.assertEqual(Foundation.NSAnchoredSearch, 8)
        self.assertEqual(Foundation.NSNumericSearch, 64)

        self.assertEqual(Foundation.NSASCIIStringEncoding, 1)
        self.assertEqual(Foundation.NSNEXTSTEPStringEncoding, 2)
        self.assertEqual(Foundation.NSJapaneseEUCStringEncoding, 3)
        self.assertEqual(Foundation.NSUTF8StringEncoding, 4)
        self.assertEqual(Foundation.NSISOLatin1StringEncoding, 5)
        self.assertEqual(Foundation.NSSymbolStringEncoding, 6)
        self.assertEqual(Foundation.NSNonLossyASCIIStringEncoding, 7)
        self.assertEqual(Foundation.NSShiftJISStringEncoding, 8)
        self.assertEqual(Foundation.NSISOLatin2StringEncoding, 9)
        self.assertEqual(Foundation.NSUnicodeStringEncoding, 10)
        self.assertEqual(Foundation.NSWindowsCP1251StringEncoding, 11)
        self.assertEqual(Foundation.NSWindowsCP1252StringEncoding, 12)
        self.assertEqual(Foundation.NSWindowsCP1253StringEncoding, 13)
        self.assertEqual(Foundation.NSWindowsCP1254StringEncoding, 14)
        self.assertEqual(Foundation.NSWindowsCP1250StringEncoding, 15)
        self.assertEqual(Foundation.NSISO2022JPStringEncoding, 21)
        self.assertEqual(Foundation.NSMacOSRomanStringEncoding, 30)
        self.assertEqual(
            Foundation.NSUTF16StringEncoding, Foundation.NSUnicodeStringEncoding
        )
        self.assertEqual(
            Foundation.NSUTF16BigEndianStringEncoding, cast_uint(0x90000100)
        )
        self.assertEqual(
            Foundation.NSUTF16LittleEndianStringEncoding, cast_uint(0x94000100)
        )
        self.assertEqual(Foundation.NSUTF32StringEncoding, cast_uint(0x8C000100))
        self.assertEqual(
            Foundation.NSUTF32BigEndianStringEncoding, cast_uint(0x98000100)
        )
        self.assertEqual(
            Foundation.NSUTF32LittleEndianStringEncoding, cast_uint(0x9C000100)
        )

        self.assertEqual(Foundation.NSStringEncodingConversionAllowLossy, 1)
        self.assertEqual(Foundation.NSStringEncodingConversionExternalRepresentation, 2)

    def testMethods(self):
        self.assertArgHasType(
            Foundation.NSString.getCharacters_range_, 0, b"o^" + objc._C_UNICHAR
        )
        self.assertArgSizeInArg(Foundation.NSString.getCharacters_range_, 0, 1)

        self.assertResultIsBOOL(Foundation.NSString.isEqualToString_)
        self.assertResultIsBOOL(Foundation.NSString.hasPrefix_)
        self.assertResultIsBOOL(Foundation.NSString.hasSuffix_)
        self.assertResultIsBOOL(Foundation.NSString.boolValue)
        self.assertArgIsOut(
            Foundation.NSString.getLineStart_end_contentsEnd_forRange_, 0
        )
        self.assertArgIsOut(
            Foundation.NSString.getLineStart_end_contentsEnd_forRange_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.getLineStart_end_contentsEnd_forRange_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.getParagraphStart_end_contentsEnd_forRange_, 0
        )
        self.assertArgIsOut(
            Foundation.NSString.getParagraphStart_end_contentsEnd_forRange_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.getParagraphStart_end_contentsEnd_forRange_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSString.dataUsingEncoding_allowLossyConversion_, 1
        )
        self.assertResultIsBOOL(Foundation.NSString.canBeConvertedToEncoding_)
        self.assertResultIsNullTerminated(Foundation.NSString.cStringUsingEncoding_)
        self.assertResultHasType(Foundation.NSString.cStringUsingEncoding_, b"^v")
        self.assertResultIsBOOL(Foundation.NSString.getCString_maxLength_encoding_)
        self.assertArgHasType(
            Foundation.NSString.getCString_maxLength_encoding_, 0, b"o^v"
        )
        self.assertArgSizeInArg(
            Foundation.NSString.getCString_maxLength_encoding_, 0, 1
        )

        self.assertResultIsBOOL(
            Foundation.NSString.getBytes_maxLength_usedLength_encoding_options_range_remainingRange_  # noqa: B950
        )
        self.assertArgHasType(
            Foundation.NSString.getBytes_maxLength_usedLength_encoding_options_range_remainingRange_,  # noqa: B950
            0,
            b"o^v",
        )
        self.assertArgSizeInArg(
            Foundation.NSString.getBytes_maxLength_usedLength_encoding_options_range_remainingRange_,  # noqa: B950
            0,
            (1, 2),
        )
        self.assertArgIsOut(
            Foundation.NSString.getBytes_maxLength_usedLength_encoding_options_range_remainingRange_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            Foundation.NSString.getBytes_maxLength_usedLength_encoding_options_range_remainingRange_,  # noqa: B950
            6,
        )

        self.assertResultHasType(
            Foundation.NSString.UTF8String, b"^" + objc._C_CHAR_AS_TEXT
        )
        self.assertResultIsNullTerminated(Foundation.NSString.UTF8String)
        self.assertResultIsNullTerminated(Foundation.NSString.availableStringEncodings)

        self.assertArgHasType(
            Foundation.NSString.initWithCharactersNoCopy_length_freeWhenDone_,
            0,
            b"n^" + objc._C_UNICHAR,
        )
        self.assertArgSizeInArg(
            Foundation.NSString.initWithCharactersNoCopy_length_freeWhenDone_, 0, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSString.initWithCharactersNoCopy_length_freeWhenDone_, 2
        )
        self.assertArgHasType(
            Foundation.NSString.initWithCharacters_length_, 0, b"n^" + objc._C_UNICHAR
        )
        self.assertArgSizeInArg(Foundation.NSString.initWithCharacters_length_, 0, 1)
        self.assertArgHasType(
            Foundation.NSString.initWithUTF8String_, 0, b"n^" + objc._C_CHAR_AS_TEXT
        )
        self.assertArgIsNullTerminated(Foundation.NSString.initWithUTF8String_, 0)
        self.assertArgIsPrintf(Foundation.NSString.initWithFormat_, 0)
        self.assertArgIsPrintf(Foundation.NSString.initWithFormat_locale_, 0)

        o = Foundation.NSString.alloc()
        self.assertArgIsIn(o.initWithBytes_length_encoding_, 0)
        self.assertArgSizeInArg(o.initWithBytes_length_encoding_, 0, 1)
        o = o.init()
        self.assertArgIsIn(
            Foundation.NSString.initWithBytesNoCopy_length_encoding_freeWhenDone_, 0
        )
        self.assertArgSizeInArg(
            Foundation.NSString.initWithBytesNoCopy_length_encoding_freeWhenDone_, 0, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSString.initWithBytesNoCopy_length_encoding_freeWhenDone_, 3
        )

        self.assertArgHasType(
            Foundation.NSString.stringWithCharacters_length_, 0, b"n^" + objc._C_UNICHAR
        )
        self.assertArgSizeInArg(Foundation.NSString.stringWithCharacters_length_, 0, 1)
        self.assertArgHasType(
            Foundation.NSString.stringWithUTF8String_, 0, b"n^" + objc._C_CHAR_AS_TEXT
        )
        self.assertArgIsNullTerminated(Foundation.NSString.stringWithUTF8String_, 0)
        self.assertArgIsPrintf(Foundation.NSString.stringWithFormat_, 0)
        self.assertArgIsPrintf(Foundation.NSString.localizedStringWithFormat_, 0)

        self.assertArgHasType(Foundation.NSString.initWithCString_encoding_, 0, b"n^t")
        self.assertArgIsNullTerminated(Foundation.NSString.initWithCString_encoding_, 0)
        self.assertArgHasType(
            Foundation.NSString.stringWithCString_encoding_, 0, b"n^t"
        )
        self.assertArgIsNullTerminated(
            Foundation.NSString.stringWithCString_encoding_, 0
        )

        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfURL_encoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfFile_encoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfURL_encoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfFile_encoding_error_, 2
        )

        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfURL_usedEncoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfURL_usedEncoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfFile_usedEncoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.initWithContentsOfFile_usedEncoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfURL_usedEncoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfURL_usedEncoding_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfFile_usedEncoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.stringWithContentsOfFile_usedEncoding_error_, 2
        )

        self.assertResultIsBOOL(
            Foundation.NSString.writeToURL_atomically_encoding_error_
        )
        self.assertArgIsBOOL(
            Foundation.NSString.writeToURL_atomically_encoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.writeToURL_atomically_encoding_error_, 3
        )
        self.assertResultIsBOOL(
            Foundation.NSString.writeToFile_atomically_encoding_error_
        )
        self.assertArgIsBOOL(
            Foundation.NSString.writeToFile_atomically_encoding_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.writeToFile_atomically_encoding_error_, 3
        )

        self.assertArgIsPrintf(Foundation.NSMutableString.appendFormat_, 0)

        self.assertResultHasType(Foundation.NSString.cString, b"^t")
        self.assertResultIsNullTerminated(Foundation.NSString.cString)
        self.assertResultHasType(Foundation.NSString.lossyCString, b"^t")
        self.assertResultIsNullTerminated(Foundation.NSString.lossyCString)

        self.assertArgHasType(Foundation.NSString.getCString_maxLength_, 0, b"o^v")
        self.assertArgSizeInArg(Foundation.NSString.getCString_maxLength_, 0, 1)

        self.assertArgHasType(
            Foundation.NSString.getCString_maxLength_range_remainingRange_, 0, b"o^v"
        )
        self.assertArgSizeInArg(
            Foundation.NSString.getCString_maxLength_range_remainingRange_, 0, 1
        )
        self.assertArgIsOut(
            Foundation.NSString.getCString_maxLength_range_remainingRange_, 3
        )

        self.assertResultIsBOOL(Foundation.NSString.writeToFile_atomically_)
        self.assertArgIsBOOL(Foundation.NSString.writeToFile_atomically_, 1)
        self.assertResultIsBOOL(Foundation.NSString.writeToURL_atomically_)
        self.assertArgIsBOOL(Foundation.NSString.writeToURL_atomically_, 1)

        self.assertArgHasType(
            Foundation.NSString.initWithCStringNoCopy_length_freeWhenDone_, 0, b"n^v"
        )
        self.assertArgSizeInArg(
            Foundation.NSString.initWithCStringNoCopy_length_freeWhenDone_, 0, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSString.initWithCStringNoCopy_length_freeWhenDone_, 2
        )
        self.assertArgHasType(Foundation.NSString.initWithCString_length_, 0, b"n^v")
        self.assertArgSizeInArg(Foundation.NSString.initWithCString_length_, 0, 1)
        self.assertArgHasType(Foundation.NSString.initWithCString_, 0, b"n^v")
        self.assertArgIsNullTerminated(Foundation.NSString.initWithCString_, 0)

        self.assertArgHasType(Foundation.NSString.stringWithCString_length_, 0, b"n^v")
        self.assertArgSizeInArg(Foundation.NSString.stringWithCString_length_, 0, 1)
        self.assertArgHasType(Foundation.NSString.stringWithCString_, 0, b"n^v")
        self.assertArgIsNullTerminated(Foundation.NSString.stringWithCString_, 0)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSStringEnumerationByLines, 0)
        self.assertEqual(Foundation.NSStringEnumerationByParagraphs, 1)
        self.assertEqual(Foundation.NSStringEnumerationByComposedCharacterSequences, 2)
        self.assertEqual(Foundation.NSStringEnumerationByWords, 3)
        self.assertEqual(Foundation.NSStringEnumerationBySentences, 4)
        self.assertEqual(Foundation.NSStringEnumerationByCaretPositions, 5)
        self.assertEqual(Foundation.NSStringEnumerationByDeletionClusters, 6)
        self.assertEqual(Foundation.NSStringEnumerationReverse, 1 << 8)
        self.assertEqual(Foundation.NSStringEnumerationSubstringNotRequired, 1 << 9)
        self.assertEqual(Foundation.NSStringEnumerationLocalized, 1 << 10)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgHasType(
            Foundation.NSString.enumerateSubstringsInRange_options_usingBlock_,
            0,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgIsBlock(
            Foundation.NSString.enumerateSubstringsInRange_options_usingBlock_,
            2,
            b"v@"
            + Foundation.NSRange.__typestr__
            + Foundation.NSRange.__typestr__
            + b"o^"
            + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSString.enumerateLinesUsingBlock_, 0, b"v@o^" + objc._C_NSBOOL
        )

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(
            Foundation.NSStringEncodingDetectionSuggestedEncodingsKey, str
        )
        self.assertIsInstance(
            Foundation.NSStringEncodingDetectionDisallowedEncodingsKey, str
        )
        self.assertIsInstance(
            Foundation.NSStringEncodingDetectionUseOnlySuggestedEncodingsKey, str
        )
        self.assertIsInstance(Foundation.NSStringEncodingDetectionAllowLossyKey, str)
        self.assertIsInstance(Foundation.NSStringEncodingDetectionFromWindowsKey, str)
        self.assertIsInstance(
            Foundation.NSStringEncodingDetectionLossySubstitutionKey, str
        )
        self.assertIsInstance(
            Foundation.NSStringEncodingDetectionLikelyLanguageKey, str
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgHasType(
            Foundation.NSString.stringEncodingForData_encodingOptions_convertedString_usedLossyConversion_,  # noqa: B950
            2,
            b"o^@",
        )
        self.assertArgHasType(
            Foundation.NSString.stringEncodingForData_encodingOptions_convertedString_usedLossyConversion_,  # noqa: B950
            3,
            b"o^Z",
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(Foundation.NSStringTransformLatinToKatakana, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToHiragana, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToHangul, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToArabic, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToHebrew, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToThai, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToCyrillic, str)
        self.assertIsInstance(Foundation.NSStringTransformLatinToGreek, str)
        self.assertIsInstance(Foundation.NSStringTransformToLatin, str)
        self.assertIsInstance(Foundation.NSStringTransformMandarinToLatin, str)
        self.assertIsInstance(Foundation.NSStringTransformHiraganaToKatakana, str)
        self.assertIsInstance(Foundation.NSStringTransformFullwidthToHalfwidth, str)
        self.assertIsInstance(Foundation.NSStringTransformToXMLHex, str)
        self.assertIsInstance(Foundation.NSStringTransformToUnicodeName, str)
        self.assertIsInstance(Foundation.NSStringTransformStripCombiningMarks, str)
        self.assertIsInstance(Foundation.NSStringTransformStripDiacritics, str)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(Foundation.NSString.localizedStandardContainsString_)
        self.assertArgIsBOOL(Foundation.NSString.stringByApplyingTransform_reverse_, 1)

        self.assertArgIsBOOL(
            Foundation.NSMutableString.applyTransform_reverse_range_updatedRange_, 1
        )
        self.assertArgIsOut(
            Foundation.NSMutableString.applyTransform_reverse_range_updatedRange_, 3
        )

    @min_os_level("11.0")
    def test_methods11_0(self):
        # XXX: exposing this not very useful.
        self.assertArgIsBlock(
            Foundation.NSString.initWithCharactersNoCopy_length_deallocator_,
            2,
            b"vn^" + objc._C_UNICHAR + objc._C_NSUInteger,
        )

        self.assertArgIsBlock(
            Foundation.NSString.initWithBytesNoCopy_length_encoding_deallocator_,
            3,
            b"v^v" + objc._C_NSUInteger,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDateIntervalFormatter(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSDateIntervalFormatterNoStyle, 0)
        self.assertEqual(Foundation.NSDateIntervalFormatterShortStyle, 1)
        self.assertEqual(Foundation.NSDateIntervalFormatterMediumStyle, 2)
        self.assertEqual(Foundation.NSDateIntervalFormatterLongStyle, 3)
        self.assertEqual(Foundation.NSDateIntervalFormatterFullStyle, 4)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSlider(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSlider.allowsTickMarkValuesOnly)
        self.assertArgIsBOOL(AppKit.NSSlider.setAllowsTickMarkValuesOnly_, 0)
        self.assertResultIsBOOL(AppKit.NSSlider.acceptsFirstMouse_)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSSlider.isVertical)
        self.assertArgIsBOOL(AppKit.NSSlider.setVertical_, 0)
        self.assertArgIsSEL(
            AppKit.NSSlider.sliderWithValue_minValue_maxValue_target_action_, 4, b"v@:@"
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSOrderedSet(TestCase):
    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.differenceFromOrderedSet_withOptions_usingEquivalenceTest_,
            2,
            b"Z@@",
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(Foundation.NSOrderedSet.isEqualToOrderedSet_)
        self.assertResultIsBOOL(Foundation.NSOrderedSet.containsObject_)
        self.assertResultIsBOOL(Foundation.NSOrderedSet.intersectsOrderedSet_)
        self.assertResultIsBOOL(Foundation.NSOrderedSet.intersectsSet_)
        self.assertResultIsBOOL(Foundation.NSOrderedSet.isSubsetOfOrderedSet_)
        self.assertResultIsBOOL(Foundation.NSOrderedSet.isSubsetOfSet_)

        self.assertArgIsBlock(
            Foundation.NSOrderedSet.enumerateObjectsUsingBlock_,
            0,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.enumerateObjectsWithOptions_usingBlock_,
            1,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.enumerateObjectsAtIndexes_options_usingBlock_,
            2,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexOfObjectPassingTest_,
            0,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexOfObjectWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexOfObjectAtIndexes_options_passingTest_,
            2,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexesOfObjectsPassingTest_,
            0,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexesOfObjectsWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexesOfObjectsAtIndexes_options_passingTest_,
            2,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSOrderedSet.indexOfObject_inSortedRange_options_usingComparator_,
            3,
            objc._C_NSInteger + b"@@",
        )

        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithOrderedSet_range_copyItems_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithArray_range_copyItems_, 2
        )
        self.assertArgIsBOOL(Foundation.NSOrderedSet.orderedSetWithArray_copyItems_, 1)
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithOrderedSet_range_copyItems_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithOrderedSet_copyItems_, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.initWithOrderedSet_range_copyItems_, 2
        )
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithOrderedSet_copyItems_, 1)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithArray_range_copyItems_, 2)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithArray_copyItems_, 1)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithSet_copyItems_, 1)

        self.assertArgIsBlock(
            Foundation.NSMutableOrderedSet.sortUsingComparator_,
            0,
            objc._C_NSInteger + b"@@",
        )
        self.assertArgIsBlock(
            Foundation.NSMutableOrderedSet.sortWithOptions_usingComparator_,
            1,
            objc._C_NSInteger + b"@@",
        )
        self.assertArgIsBlock(
            Foundation.NSMutableOrderedSet.sortRange_options_usingComparator_,
            2,
            objc._C_NSInteger + b"@@",
        )

        self.assertArgIsBlock(
            Foundation.NSOrderedSet.sortedArrayUsingComparator_,
            0,
            objc._C_NSInteger + b"@@",
        )
        self.assertArgIsBlock(
            Foundation.NSOrderedSet.sortedArrayWithOptions_usingComparator_,
            1,
            objc._C_NSInteger + b"@@",
        )

    @min_os_level("10.7")
    def testCreation(self):
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithOrderedSet_range_copyItems_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.orderedSetWithArray_range_copyItems_, 2
        )
        self.assertArgIsBOOL(Foundation.NSOrderedSet.orderedSetWithSet_copyItems_, 1)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithOrderedSet_copyItems_, 1)
        self.assertArgIsBOOL(
            Foundation.NSOrderedSet.initWithOrderedSet_range_copyItems_, 2
        )
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithArray_copyItems_, 1)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithArray_range_copyItems_, 2)
        self.assertArgIsBOOL(Foundation.NSOrderedSet.initWithSet_copyItems_, 1)

        obj = Foundation.NSOrderedSet.orderedSet()
        self.assertIsInstance(obj, Foundation.NSOrderedSet)

        obj = Foundation.NSOrderedSet.orderedSetWithObjects_count_([1, 2, 3, 4, 5], 3)
        self.assertIsInstance(obj, Foundation.NSOrderedSet)
        self.assertTrue(obj.containsObject_(1))
        self.assertTrue(obj.containsObject_(2))
        self.assertTrue(obj.containsObject_(3))
        self.assertFalse(obj.containsObject_(4))
        self.assertFalse(obj.containsObject_(5))

        obj = Foundation.NSOrderedSet.orderedSetWithObjects_(1, 2, 3)
        self.assertIsInstance(obj, Foundation.NSOrderedSet)
        self.assertTrue(obj.containsObject_(1))
        self.assertTrue(obj.containsObject_(2))
        self.assertTrue(obj.containsObject_(3))

        obj = Foundation.NSOrderedSet.alloc().initWithObjects_count_([1, 2, 3, 4, 5], 3)
        self.assertIsInstance(obj, Foundation.NSOrderedSet)
        self.assertTrue(obj.containsObject_(1))
        self.assertTrue(obj.containsObject_(2))
        self.assertTrue(obj.containsObject_(3))
        self.assertFalse(obj.containsObject_(4))
        self.assertFalse(obj.containsObject_(5))

        obj = Foundation.NSOrderedSet.alloc().initWithObjects_(1, 2, 3)
        self.assertIsInstance(obj, Foundation.NSOrderedSet)
        self.assertTrue(obj.containsObject_(1))
        self.assertTrue(obj.containsObject_(2))
        self.assertTrue(obj.containsObject_(3))

        obj = Foundation.NSMutableOrderedSet.orderedSet()
        obj.addObjects_count_([1, 2, 3, 4], 3)
        self.assertTrue(obj.containsObject_(1))
        self.assertTrue(obj.containsObject_(2))
        self.assertTrue(obj.containsObject_(3))
        self.assertFalse(obj.containsObject_(4))

        # obj = Foundation.NSMutableOrderedSet.alloc().init()
        # obj.addObjects_(1,2,3)
        # self.assertTrue(obj.containsObject_(1))
        # self.assertTrue(obj.containsObject_(2))
        # self.assertTrue(obj.containsObject_(3))

        obj.replaceObjectsInRange_withObjects_count_(
            Foundation.NSRange(0, 1), ["a", "b", "c", "d"], 3
        )
        self.assertTrue(obj.containsObject_("a"))
        self.assertFalse(obj.containsObject_(1))



import sys

import Foundation
from PyObjCTools.TestSupport import TestCase
import objc

if sys.version_info[0] == 2:
    from StringIO import StringIO

    def _str(v):
        return v


else:
    from io import StringIO

    def _str(v):
        if isinstance(v, str):
            return v
        return v.decode("ascii")


class TheadingHelperTestHelper(Foundation.NSObject):
    def init(self):
        self = objc.super(TheadingHelperTestHelper, self).init()
        if self is None:
            return None

        self.calls = []
        return self

    def performSelector_onThread_withObject_waitUntilDone_(
        self, selector, thread, value, wait
    ):
        assert isinstance(selector, bytes)
        assert isinstance(thread, Foundation.NSThread)
        assert isinstance(wait, bool)

        self.calls.append((selector, thread, value, wait))
        getattr(self, _str(selector))(value)

    def performSelector_onThread_withObject_waitUntilDone_modes_(
        self, selector, thread, value, wait, modes
    ):
        assert isinstance(selector, bytes)
        assert isinstance(thread, Foundation.NSThread)
        assert isinstance(wait, bool)

        self.calls.append((selector, thread, value, wait, modes))
        getattr(self, _str(selector))(value)

    def performSelector_withObject_afterDelay_(self, selector, value, delay):
        assert isinstance(selector, bytes)

        self.calls.append((selector, value, delay))
        getattr(self, _str(selector))(value)

    def performSelector_withObject_afterDelay_inModes_(
        self, selector, value, delay, modes
    ):
        assert isinstance(selector, bytes)

        self.calls.append((selector, value, delay, modes))
        getattr(self, _str(selector))(value)

    def performSelectorInBackground_withObject_(self, selector, value):
        self.calls.append((selector, value))
        getattr(self, _str(selector))(value)

    def performSelectorOnMainThread_withObject_waitUntilDone_(
        self, selector, value, wait
    ):
        self.calls.append((selector, value, wait))
        getattr(self, _str(selector))(value)

    def performSelectorOnMainThread_withObject_waitUntilDone_modes_(
        self, selector, value, wait, modes
    ):
        self.calls.append((selector, value, wait, modes))
        getattr(self, _str(selector))(value)

    def sel1_(self, arg):
        pass

    def sel2_(self, arg):
        return arg * 2

    def sel3_(self, arg):
        return 1 / arg


class TestThreadingHelpers(TestCase):
    def testAsyncOnThreadNoResult(self):
        # pyobjc_performSelector_onThread_withObject_waitUntilDone_
        obj = TheadingHelperTestHelper.alloc().init()
        thr = Foundation.NSThread.mainThread()

        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_(
            b"sel1:", thr, 1, False
        ),
        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_(
            b"sel2:", thr, 2, True
        )
        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_(
            b"isEqual:", thr, obj, True
        )

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", thr, (b"sel1:", 1), False),
                (b"_pyobjc_performOnThread:", thr, (b"sel2:", 2), True),
                (b"_pyobjc_performOnThread:", thr, (b"isEqual:", obj), True),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_(
                b"sel3:", thr, 0, False
            )
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", thr, (b"sel3:", 0), False)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testAsyncOnThreadNoResultModes(self):
        # pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_
        obj = TheadingHelperTestHelper.alloc().init()
        thr = Foundation.NSThread.mainThread()

        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_(
            b"sel1:", thr, 1, False, 0
        ),
        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_(
            b"sel2:", thr, 2, True, 1
        )
        obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_(
            b"isEqual:", thr, obj, True, 2
        )

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", thr, (b"sel1:", 1), False, 0),
                (b"_pyobjc_performOnThread:", thr, (b"sel2:", 2), True, 1),
                (b"_pyobjc_performOnThread:", thr, (b"isEqual:", obj), True, 2),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_(
                b"sel3:", thr, 0, False, 4
            )
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", thr, (b"sel3:", 0), False, 4)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testAsyncWithDelayNoResult(self):
        # pyobjc_performSelector_withObject_afterDelay_
        obj = TheadingHelperTestHelper.alloc().init()

        obj.pyobjc_performSelector_withObject_afterDelay_(b"sel1:", 1, 1.0)
        obj.pyobjc_performSelector_withObject_afterDelay_(b"sel2:", 2, 4.5)
        obj.pyobjc_performSelector_withObject_afterDelay_(b"isEqual:", obj, 8.5)

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", (b"sel1:", 1), 1.0),
                (b"_pyobjc_performOnThread:", (b"sel2:", 2), 4.5),
                (b"_pyobjc_performOnThread:", (b"isEqual:", obj), 8.5),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelector_withObject_afterDelay_(b"sel3:", 0, 0.5)
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", (b"sel3:", 0), 0.5)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testAsyncWithDelayNoResultModes(self):
        # pyobjc_performSelector_withObject_afterDelay_inModes_
        obj = TheadingHelperTestHelper.alloc().init()

        obj.pyobjc_performSelector_withObject_afterDelay_inModes_(b"sel1:", 1, 1.0, 0)
        obj.pyobjc_performSelector_withObject_afterDelay_inModes_(b"sel2:", 2, 4.5, 1)
        obj.pyobjc_performSelector_withObject_afterDelay_inModes_(
            b"isEqual:", obj, 8.5, 2
        )

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", (b"sel1:", 1), 1.0, 0),
                (b"_pyobjc_performOnThread:", (b"sel2:", 2), 4.5, 1),
                (b"_pyobjc_performOnThread:", (b"isEqual:", obj), 8.5, 2),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelector_withObject_afterDelay_inModes_(
                b"sel3:", 0, 0.5, 3
            )
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", (b"sel3:", 0), 0.5, 3)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testInBGNoResult(self):
        # pyobjc_performSelectorInBackground_withObject_

        obj = TheadingHelperTestHelper.alloc().init()

        obj.pyobjc_performSelectorInBackground_withObject_(b"sel1:", 1)
        obj.pyobjc_performSelectorInBackground_withObject_(b"sel2:", 2)
        obj.pyobjc_performSelectorInBackground_withObject_(b"isEqual:", obj)

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", (b"sel1:", 1)),
                (b"_pyobjc_performOnThread:", (b"sel2:", 2)),
                (b"_pyobjc_performOnThread:", (b"isEqual:", obj)),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelectorInBackground_withObject_(b"sel3:", 0)
            self.assertEqual(obj.calls, [(b"_pyobjc_performOnThread:", (b"sel3:", 0))])
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnMtNoResultWait(self):
        # pyobjc_performSelectorInBackground_withObject_waitUntilDone_

        obj = TheadingHelperTestHelper.alloc().init()

        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_(
            b"sel1:", 1, True
        )
        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_(
            b"sel2:", 2, False
        )
        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_(
            b"isEqual:", obj, True
        )

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", (b"sel1:", 1), True),
                (b"_pyobjc_performOnThread:", (b"sel2:", 2), False),
                (b"_pyobjc_performOnThread:", (b"isEqual:", obj), True),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_(
                b"sel3:", 0, False
            )
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", (b"sel3:", 0), False)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnMtNoResultWaitModes(self):
        # pyobjc_performSelectorInBackground_withObject_waitUntilDone_modes_

        obj = TheadingHelperTestHelper.alloc().init()

        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_modes_(
            b"sel1:", 1, True, 4
        )
        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_modes_(
            b"sel2:", 2, False, 5
        )
        obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_modes_(
            b"isEqual:", obj, True, 6
        )

        self.assertEqual(
            obj.calls,
            [
                (b"_pyobjc_performOnThread:", (b"sel1:", 1), True, 4),
                (b"_pyobjc_performOnThread:", (b"sel2:", 2), False, 5),
                (b"_pyobjc_performOnThread:", (b"isEqual:", obj), True, 6),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []
            obj.pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_modes_(
                b"sel3:", 0, False, 7
            )
            self.assertEqual(
                obj.calls, [(b"_pyobjc_performOnThread:", (b"sel3:", 0), False, 7)]
            )
            self.assertIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnMtWithResult(self):
        # pyobjc_performSelectorOnMainThread_withObject_
        obj = TheadingHelperTestHelper.alloc().init()

        r = obj.pyobjc_performSelectorOnMainThread_withObject_("sel2:", 3)
        self.assertEqual(r, 6)
        r = obj.pyobjc_performSelectorOnMainThread_withObject_("sel3:", 2.0)
        self.assertEqual(r, 0.5)

        self.assertEqual(
            obj.calls,
            [
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    ("sel2:", 3, [(True, 6)]),
                    True,
                ),
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    ("sel3:", 2.0, [(True, 0.5)]),
                    True,
                ),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []

            self.assertRaises(
                ZeroDivisionError,
                obj.pyobjc_performSelectorOnMainThread_withObject_,
                b"sel3:",
                0,
            )

            self.assertEqual(len(obj.calls), 1)
            self.assertEqual(obj.calls[0][0], b"_pyobjc_performOnThreadWithResult:")
            self.assertEqual(obj.calls[0][1][-1][0][0], False)
            self.assertNotIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnMtWithResultModes(self):
        obj = TheadingHelperTestHelper.alloc().init()

        r = obj.pyobjc_performSelectorOnMainThread_withObject_modes_("sel2:", 3, 1)
        self.assertEqual(r, 6)
        r = obj.pyobjc_performSelectorOnMainThread_withObject_modes_("sel3:", 2.0, 2)
        self.assertEqual(r, 0.5)

        self.assertEqual(
            obj.calls,
            [
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    ("sel2:", 3, [(True, 6)]),
                    True,
                    1,
                ),
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    ("sel3:", 2.0, [(True, 0.5)]),
                    True,
                    2,
                ),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []

            self.assertRaises(
                ZeroDivisionError,
                obj.pyobjc_performSelectorOnMainThread_withObject_modes_,
                b"sel3:",
                0,
                3,
            )

            self.assertEqual(len(obj.calls), 1)
            self.assertEqual(obj.calls[0][0], b"_pyobjc_performOnThreadWithResult:")
            self.assertEqual(obj.calls[0][1][-1][0][0], False)
            self.assertNotIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnThreadWithResult(self):
        obj = TheadingHelperTestHelper.alloc().init()
        thr = Foundation.NSThread.mainThread()

        r = obj.pyobjc_performSelector_onThread_withObject_("sel2:", thr, 3)
        self.assertEqual(r, 6)
        r = obj.pyobjc_performSelector_onThread_withObject_("sel3:", thr, 2.0)
        self.assertEqual(r, 0.5)

        self.assertEqual(
            obj.calls,
            [
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    thr,
                    ("sel2:", 3, [(True, 6)]),
                    True,
                ),
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    thr,
                    ("sel3:", 2.0, [(True, 0.5)]),
                    True,
                ),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []

            self.assertRaises(
                ZeroDivisionError,
                obj.pyobjc_performSelector_onThread_withObject_,
                b"sel3:",
                thr,
                0,
            )

            self.assertEqual(len(obj.calls), 1)
            self.assertEqual(obj.calls[0][0], b"_pyobjc_performOnThreadWithResult:")
            self.assertEqual(obj.calls[0][2][-1][0][0], False)
            self.assertNotIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr

    def testOnThreadWithResultModes(self):
        obj = TheadingHelperTestHelper.alloc().init()
        thr = Foundation.NSThread.mainThread()

        r = obj.pyobjc_performSelector_onThread_withObject_modes_("sel2:", thr, 3, 1)
        self.assertEqual(r, 6)
        r = obj.pyobjc_performSelector_onThread_withObject_modes_("sel3:", thr, 2.0, 2)
        self.assertEqual(r, 0.5)

        self.assertEqual(
            obj.calls,
            [
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    thr,
                    ("sel2:", 3, [(True, 6)]),
                    True,
                    1,
                ),
                (
                    b"_pyobjc_performOnThreadWithResult:",
                    thr,
                    ("sel3:", 2.0, [(True, 0.5)]),
                    True,
                    2,
                ),
            ],
        )

        # Raise an exception
        orig_stderr = sys.stderr
        sys.stderr = StringIO()
        try:
            obj.calls[:] = []

            self.assertRaises(
                ZeroDivisionError,
                obj.pyobjc_performSelector_onThread_withObject_modes_,
                b"sel3:",
                thr,
                0,
                3,
            )

            self.assertEqual(len(obj.calls), 1)
            self.assertEqual(obj.calls[0][0], b"_pyobjc_performOnThreadWithResult:")
            self.assertEqual(obj.calls[0][2][-1][0][0], False)
            self.assertNotIn("Traceback", sys.stderr.getvalue())
        finally:
            sys.stderr = orig_stderr



import sys
import time

import objc
import Foundation
from PyObjCTest.testhelper import PyObjC_TestClass4
from PyObjCTools.TestSupport import TestCase


class ThreadingTest(TestCase):
    def setUp(self):
        # Set a very small check interval, this will make it more likely
        # that the interpreter crashes when threading is done incorrectly.
        if sys.version_info[:2] >= (3, 2):
            self._int = sys.getswitchinterval()
            sys.setswitchinterval(0.000_000_1)
        else:
            self._int = sys.getcheckinterval()
            sys.setcheckinterval(1)

    def tearDown(self):
        if sys.version_info[:2] >= (3, 2):
            sys.setswitchinterval(self._int)
        else:
            sys.setcheckinterval(self._int)

    def testNSObjectString(self):
        class PyObjCTestThreadRunnerString(Foundation.NSObject):
            def init(self):
                self = objc.super(PyObjCTestThreadRunnerString, self).init()
                if self is None:
                    return None

                self.storage = []
                return self

            def run_(self, argument):
                Foundation.NSAutoreleasePool.alloc().init()
                self.storage.append(argument)

        myObj = PyObjCTestThreadRunnerString.alloc().init()

        Foundation.NSThread.detachNewThreadSelector_toTarget_withObject_(
            "run:", myObj, "hello world"
        )

        time.sleep(2)
        self.assertEqual(myObj.storage[0], "hello world")

    def testNSObject(self):
        class PyObjCTestThreadRunner(Foundation.NSObject):
            def run_(self, argument):
                Foundation.NSAutoreleasePool.alloc().init()
                for i in range(100):
                    argument.append(i)

        myObj = PyObjCTestThreadRunner.alloc().init()
        lst = []

        Foundation.NSThread.detachNewThreadSelector_toTarget_withObject_(
            "run:", myObj, lst
        )

        lst2 = []
        for i in range(100):
            lst2.append(i * 2)

        time.sleep(2)
        self.assertEqual(lst, list(range(100)))

    def testPyObject(self):
        import os

        class TestThreadRunner:
            def run_(self, argument):
                for i in range(100):
                    argument.append(i)

        myObj = TestThreadRunner()
        lst = []

        # Redirect stderr to avoid spurious messages when running the
        # tests.
        dupped = os.dup(2)
        fp = os.open("/dev/null", os.O_RDWR)
        os.dup2(fp, 2)
        os.close(fp)

        try:
            Foundation.NSThread.detachNewThreadSelector_toTarget_withObject_(
                "run:", myObj, lst
            )

            lst2 = []
            for i in range(100):
                lst2.append(i * 2)

            time.sleep(2)
            self.assertEqual(lst, list(range(100)))

        finally:
            os.dup2(dupped, 2)

    def testCalling(self):
        class Dummy:
            pass

        class PyObjCTestCalling(Foundation.NSObject):
            def call(self):
                return Dummy()

        my = PyObjC_TestClass4.alloc().init()
        cb = PyObjCTestCalling.alloc().init()

        Foundation.NSThread.detachNewThreadSelector_toTarget_withObject_(
            "runThread:", my, cb
        )

        time.sleep(2)

        retval = my.returnObject()
        self.assertIsInstance(retval, Dummy)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSIndexPath(TestCase):
    def testMethods(self):
        self.assertArgIsIn(Foundation.NSIndexPath.indexPathWithIndexes_length_, 0)
        self.assertArgSizeInArg(
            Foundation.NSIndexPath.indexPathWithIndexes_length_, 0, 1
        )

        self.assertArgIsIn(Foundation.NSIndexPath.initWithIndexes_length_, 0)
        self.assertArgSizeInArg(Foundation.NSIndexPath.initWithIndexes_length_, 0, 1)

        self.assertArgIsOut(Foundation.NSIndexPath.getIndexes_, 0)
        self.assertArgIsVariableSize(Foundation.NSIndexPath.getIndexes_, 0)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsOut(Foundation.NSIndexPath.getIndexes_range_, 0)
        self.assertArgSizeInArg(Foundation.NSIndexPath.getIndexes_range_, 0, 1)

    def testConvenience(self):
        path = Foundation.NSIndexPath.indexPathWithIndexes_length_([0, 1, 4], 3)

        self.assertEqual(path[0], 0)
        self.assertEqual(path[1], 1)
        self.assertEqual(path[2], 4)

        self.assertEqual(len(path), 3)

        p2 = path + 9
        self.assertEqual(len(p2), len(path) + 1)
        self.assertEqual(p2[3], 9)

        with self.assertRaises(ValueError):
            p2[1:3]



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMediaLibraryBrowserController(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSMediaLibraryAudio, 1 << 0)
        self.assertEqual(AppKit.NSMediaLibraryImage, 1 << 1)
        self.assertEqual(AppKit.NSMediaLibraryMovie, 1 << 2)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSMediaLibraryBrowserController.isVisible)
        self.assertArgIsBOOL(AppKit.NSMediaLibraryBrowserController.setVisible_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFilePromiseReceiver(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertArgIsBlock(
            AppKit.NSFilePromiseReceiver.receivePromisedFilesAtDestination_options_operationQueue_reader_,  # noqa: B950
            3,
            b"v@@",
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSScriptWhoseTestsHelper(Foundation.NSObject):
    def isEqualTo_(self, o):
        return 1

    def isLessThanOrEqualTo_(self, o):
        return 1

    def isLessThan_(self, o):
        return 1

    def isGreaterThanOrEqualTo_(self, o):
        return 1

    def isGreaterThan_(self, o):
        return 1

    def isNotEqualTo_(self, o):
        return 1

    def doesContain_(self, o):
        return 1

    def isLike_(self, o):
        return 1

    def isCaseInsensitiveLike_(self, o):
        return 1

    def scriptingIsEqualTo_(self, o):
        return 1

    def scriptingIsLessThanOrEqualTo_(self, o):
        return 1

    def scriptingIsLessThan_(self, o):
        return 1

    def scriptingIsGreaterThanOrEqualTo_(self, o):
        return 1

    def scriptingIsGreaterThan_(self, o):
        return 1

    def scriptingBeginsWith_(self, o):
        return 1

    def scriptingEndsWith_(self, o):
        return 1

    def scriptingContains_(self, o):
        return 1


class TestNSScriptWhoseTests(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSEqualToComparison, 0)
        self.assertEqual(Foundation.NSLessThanOrEqualToComparison, 1)
        self.assertEqual(Foundation.NSLessThanComparison, 2)
        self.assertEqual(Foundation.NSGreaterThanOrEqualToComparison, 3)
        self.assertEqual(Foundation.NSGreaterThanComparison, 4)
        self.assertEqual(Foundation.NSBeginsWithComparison, 5)
        self.assertEqual(Foundation.NSEndsWithComparison, 6)
        self.assertEqual(Foundation.NSContainsComparison, 7)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSScriptWhoseTest.isTrue)

        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isEqualTo_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isLessThanOrEqualTo_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isLessThan_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isGreaterThanOrEqualTo_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isGreaterThan_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isNotEqualTo_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.doesContain_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isLike_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.isCaseInsensitiveLike_)

        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingIsEqualTo_)
        self.assertResultIsBOOL(
            TestNSScriptWhoseTestsHelper.scriptingIsLessThanOrEqualTo_
        )
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingIsLessThan_)
        self.assertResultIsBOOL(
            TestNSScriptWhoseTestsHelper.scriptingIsGreaterThanOrEqualTo_
        )
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingIsGreaterThan_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingBeginsWith_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingEndsWith_)
        self.assertResultIsBOOL(TestNSScriptWhoseTestsHelper.scriptingContains_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFontAssetRequest(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSFontAssetRequestOptionUsesStandardUI, 1 << 0)

    @min_os_level("10.13")
    def testMethods(self):
        self.assertArgIsBlock(
            AppKit.NSFontAssetRequest.downloadFontAssetsWithCompletionHandler_, 0, b"Z@"
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSharingServicePickerTouchBarItem(TestCase):
    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSSharingServicePickerTouchBarItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSSharingServicePickerTouchBarItem.setEnabled_, 0)

    @min_sdk_level("10.12")
    def testProtocols10_12(self):
        objc.protocolNamed("NSSharingServicePickerTouchBarItemDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSAccessibilityCustomRotor(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSAccessibilityCustomRotorSearchDirectionPrevious, 0)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorSearchDirectionNext, 1)

        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeCustom, 0)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeAny, 1)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeAnnotation, 2)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeBoldText, 3)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeading, 4)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel1, 5)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel2, 6)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel3, 7)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel4, 8)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel5, 9)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeHeadingLevel6, 10)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeImage, 11)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeItalicText, 12)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeLandmark, 13)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeLink, 14)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeList, 15)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeMisspelledWord, 16)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeTable, 17)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeTextField, 18)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeUnderlinedText, 19)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeVisitedLink, 20)
        self.assertEqual(AppKit.NSAccessibilityCustomRotorTypeAudiograph, 21)

    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("NSAccessibilityCustomRotorItemSearchDelegate")



import pwd

from Foundation import NSMutableArray, NSObject


def getPasswords():
    a = NSMutableArray()
    for pw in pwd.getpwall():
        a.append(
            {
                "name": pw.pw_name,
                "password": pw.pw_passwd,
                "uid": str(pw.pw_uid),
                "gid": str(pw.pw_gid),
                "gecos": pw.pw_gecos,
                "home_dir": pw.pw_dir,
                "shell": pw.pw_shell,
            }
        )

    return a


class TableModelWithSearchAppDelegate(NSObject):
    def passwords(self):
        if not hasattr(self, "_cachedpasswords"):
            self._cachedpasswords = getPasswords()
        return self._cachedpasswords



from math import cos, pi, sin

# Math functions


def degToRad(deg):
    return (deg / 180.0) * pi


def radToDeg(rad):
    return (rad / pi) * 180.0


def polarToRect(polar):
    r = polar[0]
    theta = polar[1]
    return (r * cos(theta), r * sin(theta))


def bessel(z, t=0.00001):
    j = 1
    jn = 1
    zz4 = z * z / 4
    for k in range(1, 100):
        jn *= -1 * zz4 / (k * k)
        j += jn

        if jn < 0:
            if jn > t:
                break
        else:
            if jn < t:
                break
    return j



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSPortCoder(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSPortCoder.isBycopy)
        self.assertResultIsBOOL(Foundation.NSPortCoder.isByref)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSURLCredential(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSURLCredentialPersistenceNone, 0)
        self.assertEqual(Foundation.NSURLCredentialPersistenceForSession, 1)
        self.assertEqual(Foundation.NSURLCredentialPersistencePermanent, 2)
        self.assertEqual(Foundation.NSURLCredentialPersistenceSynchronizable, 3)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSURLCredential.hasPassword)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPrintPanel(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSPrintPanelAccessorizing")

    def testConstants(self):
        self.assertEqual(AppKit.NSPrintPanelShowsCopies, 0x01)
        self.assertEqual(AppKit.NSPrintPanelShowsPageRange, 0x02)
        self.assertEqual(AppKit.NSPrintPanelShowsPaperSize, 0x04)
        self.assertEqual(AppKit.NSPrintPanelShowsOrientation, 0x08)
        self.assertEqual(AppKit.NSPrintPanelShowsScaling, 0x10)
        self.assertEqual(AppKit.NSPrintPanelShowsPageSetupAccessory, 0x100)
        self.assertEqual(AppKit.NSPrintPanelShowsPreview, 0x20000)

        self.assertIsInstance(AppKit.NSPrintPhotoJobStyleHint, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSPrintPanelAccessorySummaryItemNameKey, str)
        self.assertIsInstance(
            AppKit.NSPrintPanelAccessorySummaryItemDescriptionKey, str
        )

    def testMethods(self):
        self.assertArgIsSEL(
            AppKit.NSPrintPanel.beginSheetWithPrintInfo_modalForWindow_delegate_didEndSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSPrintPanel.beginSheetWithPrintInfo_modalForWindow_delegate_didEndSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSPrintPanelShowsPrintSelection, 1 << 5)

        self.assertIsInstance(AppKit.NSPrintAllPresetsJobStyleHint, str)
        self.assertIsInstance(AppKit.NSPrintNoPresetsJobStyleHint, str)



#
#  GraphicsView.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html


import objc
from Circle import Circle
from Cocoa import (
    NSBezierPath,
    NSColor,
    NSDrawLightBezel,
    NSIndexSet,
    NSInsetRect,
    NSIntersectsRect,
    NSKeyValueChangeNewKey,
    NSKeyValueChangeOldKey,
    NSKeyValueObservingOptionNew,
    NSKeyValueObservingOptionOld,
    NSMakeRect,
    NSNotFound,
    NSShiftKeyMask,
    NSUnionRect,
    NSView,
)
from objc import super


PropertyObservationContext = 1091
GraphicsObservationContext = 1092
SelectionIndexesObservationContext = 1093


class GraphicsView(NSView):
    graphicsContainer = objc.ivar("graphicsContainer")
    graphicsKeyPath = objc.ivar("graphicsKeyPath")

    selectionIndexesContainer = objc.ivar(
        "selectionIndexesContainer"
    )  # GraphicsArrayController
    selectionIndexesKeyPath = objc.ivar("selectionIndexesKeyPath")

    oldGraphics = objc.ivar("oldGraphics")

    def exposedBindings(self):
        return ["graphics", "selectedObjects"]

    def initWithFrame_(self, frameRect):
        return super().initWithFrame_(frameRect)

    def graphics(self):
        if not self.graphicsContainer:
            return None
        return self.graphicsContainer.valueForKeyPath_(self.graphicsKeyPath)

    def selectionIndexes(self):
        if not self.selectionIndexesContainer:
            return None
        return self.selectionIndexesContainer.valueForKeyPath_(
            self.selectionIndexesKeyPath
        )

    def startObservingGraphics_(self, graphics):
        if not graphics:
            return
        # Register to observe each of the new graphics, and
        # each of their observable properties -- we need old and new
        # values for drawingBounds to figure out what our dirty rect
        for newGraphic in graphics:
            # Register as observer for all the drawing-related properties
            newGraphic.addObserver_forKeyPath_options_context_(
                self,
                "drawingBounds",
                (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld),
                PropertyObservationContext,
            )
            keys = Circle.keysForNonBoundsProperties()
            for key in keys:
                newGraphic.addObserver_forKeyPath_options_context_(
                    self, key, 0, PropertyObservationContext
                )

    def stopObservingGraphics_(self, graphics):
        if graphics is None:
            return
        for graphic in graphics:
            for key in graphic.class__().keysForNonBoundsProperties():
                graphic.removeObserver_forKeyPath_(self, key)
            graphic.removeObserver_forKeyPath_(self, "drawingBounds")

    def bind_toObject_withKeyPath_options_(
        self, bindingName, observableObject, observableKeyPath, options
    ):
        if bindingName == "graphics":
            self.graphicsContainer = observableObject
            self.graphicsKeyPath = observableKeyPath
            self.graphicsContainer.addObserver_forKeyPath_options_context_(
                self,
                self.graphicsKeyPath,
                (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld),
                GraphicsObservationContext,
            )
            self.startObservingGraphics_(self.graphics())

        elif bindingName == "selectionIndexes":
            self.selectionIndexesContainer = observableObject
            self.selectionIndexesKeyPath = observableKeyPath
            self.selectionIndexesContainer.addObserver_forKeyPath_options_context_(
                self,
                self.selectionIndexesKeyPath,
                0,
                SelectionIndexesObservationContext,
            )
        self.setNeedsDisplay_(True)

    def unbind_(self, bindingName):
        if bindingName == "graphics":
            self.graphicsContainer.removeObserver_forKeyPath_(
                self, self.graphicsKeyPath
            )
            self.graphicsContainer = None
            self.graphicsKeyPath = None
        if bindingName == "selectionIndexes":
            self.selectionIndexesContainer.removeObserver_forKeyPath_(
                self, self.selectionIndexesKeyPath
            )
            self.seletionIndexesContainer = None
            self.selectionIndexesKeyPath = None
        self.setNeedsDisplay_(True)

    def observeValueForKeyPath_ofObject_change_context_(
        self, keyPath, an_object, change, context
    ):
        if context == GraphicsObservationContext:
            # Should be able to use
            # NSArray *oldGraphics = [change objectForKey:NSKeyValueChangeOldKey];
            # etc. but the dictionary doesn't contain old and new arrays...??
            newGraphics = set(an_object.valueForKeyPath_(self.graphicsKeyPath))
            onlyNew = newGraphics - set(self.oldGraphics or [])
            self.startObservingGraphics_(onlyNew)

            if self.oldGraphics:
                removed = set(self.oldGraphics) - newGraphics
                self.stopObservingGraphics_(removed)

            self.oldGraphics = newGraphics

            # could check drawingBounds of old and new, but...
            self.setNeedsDisplay_(True)
            return

        if context == PropertyObservationContext:
            updateRect = (0,)
            # Note: for Circle, drawingBounds is a dependent key of all the other
            # property keys except color, so we'll get this anyway...
            if keyPath == "drawingBounds":
                newBounds = change.objectForKey_(NSKeyValueChangeNewKey)
                oldBounds = change.objectForKey_(NSKeyValueChangeOldKey)
                updateRect = NSUnionRect(newBounds, oldBounds)
            else:
                updateRect = an_object.drawingBounds()
            updateRect = NSMakeRect(
                updateRect.origin.x - 1.0,
                updateRect.origin.y - 1.0,
                updateRect.size.width + 2.0,
                updateRect.size.height + 2.0,
            )
            self.setNeedsDisplay_(True)
            return

        if context == SelectionIndexesObservationContext:
            self.setNeedsDisplay_(True)
            return

    def drawRect_(self, rect):
        myBounds = self.bounds()
        NSDrawLightBezel(myBounds, myBounds)  # AppKit Function
        clipRect = NSBezierPath.bezierPathWithRect_(NSInsetRect(myBounds, 2.0, 2.0))
        clipRect.addClip()

        # Draw graphics
        graphicsArray = self.graphics()
        if graphicsArray:
            for graphic in graphicsArray:
                graphicDrawingBounds = graphic.drawingBounds()
                if NSIntersectsRect(rect, graphicDrawingBounds):
                    graphic.drawInView_(self)

        # Draw a red box around items in the current selection.
        # Selection should be handled by the graphic, but this is a
        # shortcut simply for display.

        currentSelectionIndexes = self.selectionIndexes()
        if currentSelectionIndexes is not None:
            path = NSBezierPath.bezierPath()
            index = currentSelectionIndexes.firstIndex()
            while index != NSNotFound:
                graphicDrawingBounds = graphicsArray[index].drawingBounds()
                if NSIntersectsRect(rect, graphicDrawingBounds):
                    path.appendBezierPathWithRect_(graphicDrawingBounds)
                index = currentSelectionIndexes.indexGreaterThanIndex_(index)

            NSColor.redColor().set()
            path.setLineWidth_(1.5)
            path.stroke()

        # Fairly simple just to illustrate the point

    def mouseDown_(self, event):
        # find out if we hit anything
        p = self.convertPoint_fromView_(event.locationInWindow(), None)
        for aGraphic in self.graphics():
            if aGraphic.hitTest_isSelected_(p, False):
                break

        else:
            aGraphic = None

        # if no graphic hit, then if extending selection do nothing
        # else set selection to nil
        if aGraphic is None:
            if not event.modifierFlags() & NSShiftKeyMask:
                self.selectionIndexesContainer.setValue_forKeyPath_(
                    None, self.selectionIndexesKeyPath
                )
            return

        # graphic hit
        # if not extending selection (Shift key down) then set
        # selection to this graphic
        # if extending selection, then:
        # - if graphic in selection remove it
        # - if not in selection add it
        graphicIndex = self.graphics().index(aGraphic)
        if not event.modifierFlags() & NSShiftKeyMask:
            selection = NSIndexSet.indexSetWithIndex_(graphicIndex)
        else:
            if self.selectionIndexes().containsIndex_(graphicIndex):
                selection = self.selectionIndexes().mutableCopy()
                selection.removeIndex_(graphicIndex)
            else:
                selection = self.selectionIndexes().mutableCopy()
                selection.addIndex_(graphicIndex)

        self.selectionIndexesContainer.setValue_forKeyPath_(
            selection, self.selectionIndexesKeyPath
        )


GraphicsView.exposeBinding_("graphics")
GraphicsView.exposeBinding_("selectionIndexes")



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSErrors(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSTextLineTooLongException, str)
        self.assertIsInstance(AppKit.NSTextNoSelectionException, str)
        self.assertIsInstance(AppKit.NSWordTablesWriteException, str)
        self.assertIsInstance(AppKit.NSWordTablesReadException, str)
        self.assertIsInstance(AppKit.NSTextReadException, str)
        self.assertIsInstance(AppKit.NSTextWriteException, str)
        self.assertIsInstance(AppKit.NSPasteboardCommunicationException, str)
        self.assertIsInstance(AppKit.NSPrintingCommunicationException, str)
        self.assertIsInstance(AppKit.NSAbortModalException, str)
        self.assertIsInstance(AppKit.NSAbortPrintingException, str)
        self.assertIsInstance(AppKit.NSIllegalSelectorException, str)
        self.assertIsInstance(AppKit.NSAppKitVirtualMemoryException, str)
        self.assertIsInstance(AppKit.NSBadRTFDirectiveException, str)
        self.assertIsInstance(AppKit.NSBadRTFFontTableException, str)
        self.assertIsInstance(AppKit.NSBadRTFStyleSheetException, str)
        self.assertIsInstance(AppKit.NSTypedStreamVersionException, str)
        self.assertIsInstance(AppKit.NSTIFFException, str)
        self.assertIsInstance(AppKit.NSPrintPackageException, str)
        self.assertIsInstance(AppKit.NSBadRTFColorTableException, str)
        self.assertIsInstance(AppKit.NSDraggingException, str)
        self.assertIsInstance(AppKit.NSColorListIOException, str)
        self.assertIsInstance(AppKit.NSColorListNotEditableException, str)
        self.assertIsInstance(AppKit.NSBadBitmapParametersException, str)
        self.assertIsInstance(AppKit.NSWindowServerCommunicationException, str)
        self.assertIsInstance(AppKit.NSFontUnavailableException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeNotFoundException, str)
        self.assertIsInstance(AppKit.NSPPDParseException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeStackOverflowException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeStackUnderflowException, str)
        self.assertIsInstance(AppKit.NSRTFPropertyStackOverflowException, str)
        self.assertIsInstance(AppKit.NSAppKitIgnoredException, str)
        self.assertIsInstance(AppKit.NSBadComparisonException, str)
        self.assertIsInstance(AppKit.NSImageCacheException, str)
        self.assertIsInstance(AppKit.NSNibLoadingException, str)
        self.assertIsInstance(AppKit.NSBrowserIllegalDelegateException, str)
        self.assertIsInstance(AppKit.NSAccessibilityException, str)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSHTTPCookieStorage(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSHTTPCookieAcceptPolicyAlways, 0)
        self.assertEqual(Foundation.NSHTTPCookieAcceptPolicyNever, 1)
        self.assertEqual(
            Foundation.NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain, 2
        )

        self.assertIsInstance(
            Foundation.NSHTTPCookieManagerAcceptPolicyChangedNotification, str
        )
        self.assertIsInstance(
            Foundation.NSHTTPCookieManagerCookiesChangedNotification, str
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSHTTPCookieStorage.getCookiesForTask_completionHandler_,
            1,
            b"v@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMenuItem(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSMenuItem.usesUserKeyEquivalents)
        self.assertArgIsBOOL(AppKit.NSMenuItem.setUsesUserKeyEquivalents_, 0)
        self.assertResultIsBOOL(AppKit.NSMenuItem.hasSubmenu)
        self.assertResultIsBOOL(AppKit.NSMenuItem.isSeparatorItem)
        self.assertResultIsBOOL(AppKit.NSMenuItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSMenuItem.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSMenuItem.isAlternate)
        self.assertArgIsBOOL(AppKit.NSMenuItem.setAlternate_, 0)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSMenuItem.isHighlighted)
        self.assertResultIsBOOL(AppKit.NSMenuItem.isHidden)
        self.assertArgIsBOOL(AppKit.NSMenuItem.setHidden_, 0)
        self.assertResultIsBOOL(AppKit.NSMenuItem.isHiddenOrHasHiddenAncestor)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSMenuItem.allowsKeyEquivalentWhenHidden)
        self.assertArgIsBOOL(AppKit.NSMenuItem.setAllowsKeyEquivalentWhenHidden_, 0)

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertResultIsBOOL(
            AppKit.NSMenuItem.allowsAutomaticKeyEquivalentLocalization
        )
        self.assertArgIsBOOL(
            AppKit.NSMenuItem.setAllowsAutomaticKeyEquivalentLocalization_, 0
        )

        self.assertResultIsBOOL(AppKit.NSMenuItem.allowsAutomaticKeyEquivalentMirroring)
        self.assertArgIsBOOL(
            AppKit.NSMenuItem.setAllowsAutomaticKeyEquivalentMirroring_, 0
        )



import datetime
import objc

import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCFCalendarVariadic(TestCase):
    def testTypes(self):
        cls = None
        try:
            cls = objc.lookUpClass("NSCFCalendar")
        except objc.error:
            cls = objc.lookUpClass("__NSCFCalendar")

        if cls is None:
            self.assertIsCFType(CoreFoundation.CFCalendarRef)

        else:
            self.assertIs(CoreFoundation.CFCalendarRef, cls)

    def testCFCalendarComposeAbsoluteTime(self):
        calendar = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFGregorianCalendar
        )
        self.assertIsInstance(calendar, CoreFoundation.CFCalendarRef)

        success, at = CoreFoundation.CFCalendarComposeAbsoluteTime(calendar, None, b"")
        self.assertEqual(success, True)
        self.assertIsInstance(at, float)

        success, at = CoreFoundation.CFCalendarComposeAbsoluteTime(
            calendar, None, b"yMdHms", 1965, 1, 6, 14, 10, 0
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at, float)

    def testCFCalendarAddComponents(self):
        calendar = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFGregorianCalendar
        )
        self.assertIsInstance(calendar, CoreFoundation.CFCalendarRef)

        success, at = CoreFoundation.CFCalendarComposeAbsoluteTime(
            calendar, None, b"yMdHms", 1965, 1, 6, 14, 10, 0
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at, float)

        success, at2 = CoreFoundation.CFCalendarAddComponents(
            calendar, at, 0, b"yH", 2, 3
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at2, float)

        success, y, H = CoreFoundation.CFCalendarGetComponentDifference(
            calendar, at, at2, 0, b"yH"
        )
        self.assertEqual(success, True)
        self.assertEqual(y, 2)
        self.assertEqual(H, 3)

    def testCFCalendarDecomposeAbsoluteTime(self):
        calendar = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFGregorianCalendar
        )
        self.assertTrue(calendar is not None)

        success, at = CoreFoundation.CFCalendarComposeAbsoluteTime(
            calendar, None, b"yMdHms", 1965, 1, 6, 14, 10, 0
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at, float)

        success, y, M, d, H, m, s = CoreFoundation.CFCalendarDecomposeAbsoluteTime(
            calendar, at, b"yMdHms"
        )
        self.assertEqual(y, 1965)
        self.assertEqual(M, 1)
        self.assertEqual(d, 6)
        self.assertEqual(H, 14)
        self.assertEqual(m, 10)
        self.assertEqual(s, 0)

    def testCFCalendarGetComponentDifference(self):
        calendar = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFGregorianCalendar
        )
        self.assertTrue(calendar is not None)

        success, at1 = CoreFoundation.CFCalendarComposeAbsoluteTime(
            calendar, None, b"yMdHms", 1965, 1, 6, 14, 10, 0
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at1, float)

        success, at2 = CoreFoundation.CFCalendarComposeAbsoluteTime(
            calendar, None, b"yMdHms", 1967, 2, 6, 14, 10, 0
        )
        self.assertEqual(success, True)
        self.assertIsInstance(at2, float)

        success, y, M = CoreFoundation.CFCalendarGetComponentDifference(
            calendar, at1, at2, 0, b"yM"
        )
        self.assertEqual(success, True)
        self.assertEqual(y, 2)
        self.assertEqual(M, 1)

    def testTypeID(self):
        v = CoreFoundation.CFCalendarGetTypeID()
        self.assertIsInstance(v, int)

    def testCreation(self):
        cal = CoreFoundation.CFCalendarCopyCurrent()
        self.assertIsInstance(cal, CoreFoundation.CFCalendarRef)
        cal = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFBuddhistCalendar
        )
        self.assertIsInstance(cal, CoreFoundation.CFCalendarRef)

    def testInspection(self):
        cal = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFGregorianCalendar
        )
        self.assertIsInstance(cal, CoreFoundation.CFCalendarRef)
        name = CoreFoundation.CFCalendarGetIdentifier(cal)
        self.assertEqual(name, CoreFoundation.kCFGregorianCalendar)
        locale = CoreFoundation.CFCalendarCopyLocale(cal)
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)
        timezone = CoreFoundation.CFCalendarCopyTimeZone(cal)
        self.assertIsInstance(timezone, CoreFoundation.CFTimeZoneRef)
        weekday = CoreFoundation.CFCalendarGetFirstWeekday(cal)
        self.assertLessEqual(0 <= weekday, 7)
        num = CoreFoundation.CFCalendarGetMinimumDaysInFirstWeek(cal)
        self.assertLessEqual(0 <= num, 7)
        rng = CoreFoundation.CFCalendarGetMinimumRangeOfUnit(
            cal, CoreFoundation.kCFCalendarUnitEra
        )
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        rng = CoreFoundation.CFCalendarGetMaximumRangeOfUnit(
            cal, CoreFoundation.kCFCalendarUnitEra
        )
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        m = datetime.date.today()
        if m.month in (1, 3, 5, 7, 8, 10, 12):
            monthLength = 31
        elif m.month in (4, 6, 9, 11):
            monthLength = 30
        else:
            if m.year % 4 == 0:
                # Yes this is wrong, but the next time this fails in
                # in 2100.
                monthLength = 29
            else:
                monthLength = 28
        rng = CoreFoundation.CFCalendarGetRangeOfUnit(
            cal,
            CoreFoundation.kCFCalendarUnitDay,
            CoreFoundation.kCFCalendarUnitMonth,
            CoreFoundation.CFAbsoluteTimeGetCurrent(),
        )
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        self.assertEqual(rng.location, 1)
        self.assertEqual(rng.length, monthLength)

        v = CoreFoundation.CFCalendarGetOrdinalityOfUnit(
            cal,
            CoreFoundation.kCFCalendarUnitDay,
            CoreFoundation.kCFCalendarUnitYear,
            CoreFoundation.CFAbsoluteTimeGetCurrent(),
        )
        self.assertIsInstance(v, int)
        ok, startp, tip = CoreFoundation.CFCalendarGetTimeRangeOfUnit(
            cal,
            CoreFoundation.kCFCalendarUnitDay,
            CoreFoundation.CFAbsoluteTimeGetCurrent(),
            None,
            None,
        )
        self.assertIs(ok, True)
        self.assertIsInstance(startp, float)
        self.assertIsInstance(tip, float)
        self.assertIn(tip, (86400.0, 90000.0, 82800))  # 1 day, remove DST, add DST

    def testMutation(self):
        cal = CoreFoundation.CFCalendarCreateWithIdentifier(
            None, CoreFoundation.kCFBuddhistCalendar
        )

        loc = CoreFoundation.CFLocaleCreate(None, "mr_IN")
        self.assertIsInstance(loc, CoreFoundation.CFLocaleRef)
        id1 = CoreFoundation.CFLocaleGetIdentifier(loc)

        orig_loc = CoreFoundation.CFCalendarCopyLocale(cal)
        self.assertIsInstance(orig_loc, CoreFoundation.CFLocaleRef)
        orig_id = CoreFoundation.CFLocaleGetIdentifier(orig_loc)
        CoreFoundation.CFCalendarSetLocale(cal, loc)
        new_loc = CoreFoundation.CFCalendarCopyLocale(cal)
        self.assertIsInstance(new_loc, CoreFoundation.CFLocaleRef)
        new_id = CoreFoundation.CFLocaleGetIdentifier(new_loc)

        self.assertEqual(new_id, id1)
        self.assertNotEqual(orig_id, id1)
        tz = CoreFoundation.CFTimeZoneCreateWithName(None, "Pacific/Wallis", True)
        self.assertIsInstance(tz, CoreFoundation.CFTimeZoneRef)
        orig_zone = CoreFoundation.CFCalendarCopyTimeZone(cal)
        self.assertIsInstance(orig_zone, CoreFoundation.CFTimeZoneRef)
        CoreFoundation.CFCalendarSetTimeZone(cal, tz)
        new_zone = CoreFoundation.CFCalendarCopyTimeZone(cal)
        self.assertIsInstance(new_zone, CoreFoundation.CFTimeZoneRef)
        self.assertEqual(CoreFoundation.CFTimeZoneGetName(new_zone), "Pacific/Wallis")
        weekday = CoreFoundation.CFCalendarGetFirstWeekday(cal)
        weekday = weekday + 2 % 7
        CoreFoundation.CFCalendarSetFirstWeekday(cal, weekday)
        new = CoreFoundation.CFCalendarGetFirstWeekday(cal)
        self.assertEqual(new, weekday)
        num = CoreFoundation.CFCalendarGetMinimumDaysInFirstWeek(cal)
        if num == 1:
            num = 2
        else:
            num = 1

        CoreFoundation.CFCalendarSetMinimumDaysInFirstWeek(cal, num)
        num2 = CoreFoundation.CFCalendarGetMinimumDaysInFirstWeek(cal)
        self.assertEqual(num2, num)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFCalendarUnitEra, 1 << 1)
        self.assertEqual(CoreFoundation.kCFCalendarUnitYear, 1 << 2)
        self.assertEqual(CoreFoundation.kCFCalendarUnitMonth, 1 << 3)
        self.assertEqual(CoreFoundation.kCFCalendarUnitDay, 1 << 4)
        self.assertEqual(CoreFoundation.kCFCalendarUnitHour, 1 << 5)
        self.assertEqual(CoreFoundation.kCFCalendarUnitMinute, 1 << 6)
        self.assertEqual(CoreFoundation.kCFCalendarUnitSecond, 1 << 7)
        self.assertEqual(CoreFoundation.kCFCalendarUnitWeek, 1 << 8)
        self.assertEqual(CoreFoundation.kCFCalendarUnitWeekday, 1 << 9)
        self.assertEqual(CoreFoundation.kCFCalendarUnitWeekdayOrdinal, 1 << 10)
        self.assertEqual(CoreFoundation.kCFCalendarComponentsWrap, 1 << 0)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreFoundation.kCFCalendarUnitQuarter, 1 << 11)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(CoreFoundation.kCFCalendarUnitWeekOfMonth, 1 << 12)
        self.assertEqual(CoreFoundation.kCFCalendarUnitWeekOfYear, 1 << 13)
        self.assertEqual(CoreFoundation.kCFCalendarUnitYearForWeekOfYear, 1 << 14)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSOpenGLView(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSOpenGLView.wantsBestResolutionOpenGLSurface)
        self.assertArgIsBOOL(
            AppKit.NSOpenGLView.setWantsBestResolutionOpenGLSurface_, 0
        )



import pprint

from PyObjCTools.Conversion import propertyListFromPythonCollection

input_value = {
    "a": [1, 2, 3, 4],
    "b": "c",
    "d": 3,
    "e": None,
    "f": [1, {"2": 3, "a": "b"}, [3, 4]],
    "g": {"h": "i"},
    "j": (1, 2, 3, "k", "l"),
}

print("Converting (Python Collection):")
pprint.pprint(input_value)

convertedCollection = propertyListFromPythonCollection(input_value)

print()
print()
print("Converted (ObjC collection):")
print("%s" % convertedCollection.description())
